import os

from mosamaticdesktop.core.loaders.loader import Loader
from mosamaticdesktop.core.loaders.fileloader import FileLoader
from mosamaticdesktop.core.loaders.dicomfileloader import DicomFileLoader
from mosamaticdesktop.core.data.dicomseriesdata import DicomSeriesData


class DicomSeriesLoader(Loader, FileLoader):
    def __init__(self):
        self._dir_path = None

    # implements(FileSetLoader)
    def path(self):
        return self._dir_path
    
    # implements(FileSetLoader)
    def set_path(self, path):
        self._dir_path = path

    # implements(Loader)
    def load(self):
        if self.path():
            data = DicomSeriesData()
            data.set_path(self.path())
            object = []
            for f in os.listdir(self.path()):
                f_path = os.path.join(self.path(), f)
                loader = DicomFileLoader()
                loader.set_path(f_path)
                file_data = loader.load() # Returns None if not DICOM
                if file_data:
                    object.append(file_data)
            # Sort DICOM objects by instance number
            object_sorted = sorted(object, key=lambda item: int(item.object().get('InstanceNumber')))
            data.set_object(object_sorted)
            return data
        raise RuntimeError('Directory path not set')


import os
import webbrowser

from PySide6.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QPushButton,
    QLabel,
)
from PySide6.QtCore import Qt
from PySide6.QtGui import QPixmap

from mosamaticdesktop.ui.mainwindow import MainWindow
from mosamaticdesktop.ui.utils import resource_path, set_opacity, version

RESOURCES_IMAGES_DIR = 'mosamaticdesktop/resources/images'
BACKGROUND_IMAGE_FILE_NAME = 'body-composition.jpg'
BACKGROUND_IMAGE_OPACITY = 0.4
WINDOW_W = 600
WINDOW_H = 300
DONATE_URL = 'https://rbeesoft.nl/wordpress/'


class SplashScreen(QWidget):
    def __init__(self):
        super(SplashScreen, self).__init__()
        self._main = None
        self._background_label = None
        self._background_pixmap = None
        self._title_label = None
        self._sub_text_label = None
        self._start_app_button = None
        self._donate_button = None
        self._close_button = None
        self.init_layout()

    def main(self):
        if not self._main:
            self._main = MainWindow()
        return self._main
    
    def background_label(self):
        if not self._background_label:
            self._background_label = QLabel(self)
            # self._background_label.setPixmap(self.background_pixmap())
            # self._background_label.setGeometry(0, 0, self.width(), self.height())
            # self._background_label.lower()
        return self._background_label
    
    def background_pixmap(self):
        if not self._background_pixmap:
            self._background_pixmap = QPixmap(resource_path(os.path.join(
                RESOURCES_IMAGES_DIR, BACKGROUND_IMAGE_FILE_NAME,
            ))).scaled(self.size(), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
            self._background_pixmap = set_opacity(self._background_pixmap, BACKGROUND_IMAGE_OPACITY)
        return self._background_pixmap
    
    def title_label(self):
        if not self._title_label:
            self._title_label = QLabel(f'Mosamatic Desktop {version()}')
            self._title_label.setStyleSheet('color: rgb(64, 64, 64); font-weight: bold; font-size: 32pt;')
            self._title_label.setAlignment(Qt.AlignCenter)
        return self._title_label
    
    def sub_text_label(self):
        if not self._sub_text_label:
            message = 'This software is for research only'
            self._sub_text_label = QLabel(message)
            self._sub_text_label.setStyleSheet('color: rgb(64, 64, 64); font-style: italic; font-size: 10pt;')
            self._sub_text_label.setAlignment(Qt.AlignCenter)
        return self._sub_text_label
    
    def start_app_button(self):
        if not self._start_app_button:
            self._start_app_button = QPushButton('Start app')
            self._start_app_button.clicked.connect(self.handle_start_app_button)
        return self._start_app_button
    
    def donate_button(self):
        if not self._donate_button:
            self._donate_button = QPushButton('If you wish to support us, please consider a donation by clicking here!')
            self._donate_button.setStyleSheet('background-color: blue; color: white; font-weight: bold;')
            self._donate_button.clicked.connect(self.handle_donate_button)
        return self._donate_button
    
    def close_button(self):
        if not self._close_button:
            self._close_button = QPushButton('Quit')
            self._close_button.clicked.connect(self.handle_close_button)
        return self._close_button
    
    # LAYOUT

    def init_layout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.title_label())
        layout.addWidget(self.sub_text_label())
        layout.addWidget(self.start_app_button())
        # layout.addWidget(self.donate_button())
        layout.addWidget(self.close_button())
        self.setLayout(layout)
        self.setFixedSize(WINDOW_W, WINDOW_H)
        self.setWindowFlags(Qt.FramelessWindowHint)
    
    # EVENT HANDLERS

    def handle_start_app_button(self):
        self.close()
        self.main().show()

    def handle_donate_button(self):
        webbrowser.open(DONATE_URL)

    def handle_close_button(self):
        self.close()

    def resizeEvent(self, event):
        super().resizeEvent(event)
        # scaled = self.background_pixmap().scaled(self.size(), Qt.KeepAspectRatioByExpanding, Qt.SmoothTransformation)
        # self.background_label().setPixmap(scaled)
        # self.background_label().setGeometry(0, 0, self.width(), self.height())



import pendulum


def create_name_with_timestamp(prefix: str='') -> str:
    tz = pendulum.local_timezone()
    timestamp = pendulum.now(tz).strftime('%Y%m%d%H%M%S%f')[:17]
    if prefix != '' and not prefix.endswith('-'):
        prefix = prefix + '-'
    name = f'{prefix}{timestamp}'
    return name


import pydicom


from mosamaticdesktop.core.loaders.loader import Loader
from mosamaticdesktop.core.loaders.fileloader import FileLoader
from mosamaticdesktop.core.data.dicomfiledata import DicomFileData
from mosamatic.utils import (
    is_dicom, 
    load_dicom, 
    is_jpeg2000_compressed,
)


class DicomFileLoader(Loader, FileLoader):
    def __init__(self):
        self._file_path = None

    # implements(FileLoader)
    def path(self):
        return self._file_path
    
    # implements(FileLoader)
    def set_path(self, path):
        self._file_path = path

    # implements(Loader)
    def load(self):
        if self.path():
            data = DicomFileData()
            data.set_path(self.path())
            if is_dicom(self.path()):
                p = load_dicom(self.path())
                if is_jpeg2000_compressed(p):
                    p.decompress()
                data.set_object(pydicom.dcmread(self.path()))
                return data
            return None
        raise RuntimeError('File path not set')


import sys
import traceback

from PySide6 import QtWidgets

from mosamaticdesktop.ui.settings import Settings
from mosamaticdesktop.core.utils.logmanager import LogManager
from mosamaticdesktop.ui.components.splashscreen import SplashScreen

WINDOW_TITLE = 'Mosamatic Desktop'

LOG = LogManager()


def main():
    settings = Settings()
    application_name = settings.get(WINDOW_TITLE)
    QtWidgets.QApplication.setApplicationName(application_name)
    app = QtWidgets.QApplication(sys.argv)
    app.setApplicationName(application_name)
    # main_window = MainWindow()
    # main_window.show()
    try:
        splash = SplashScreen()
        splash.show()
        sys.exit(app.exec())
    except Exception as e:
        LOG.error(str(e))
        LOG.error(traceback.format_exc())
        sys.exit(1)

if __name__ == '__main__':
    main()


import webbrowser

from PySide6.QtWidgets import (
    QWidget,
    QLabel,
    QPushButton,
    QVBoxLayout,
    QDockWidget,
)

from mosamaticdesktop.ui.settings import Settings
from mosamaticdesktop.ui.panels.stackedpanel import StackedPanel
from mosamaticdesktop.core.utils.logmanager import LogManager

LOG = LogManager()

PANEL_NAME = 'mainpanel'
DONATE_URL = 'https://rbeesoft.nl/wordpress/'


class MainPanel(QDockWidget):
    def __init__(self, parent):
        super(MainPanel, self).__init__(parent)
        self._settings = None
        self._title_label = None
        self._donate_button = None
        self._stacked_panel = None
        self._panels = {}
        self.init_layout()

    def init_layout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.title_label())
        # layout.addWidget(self.donate_button())
        layout.addWidget(self.stacked_panel())
        container = QWidget()
        container.setLayout(layout)
        self.setObjectName(PANEL_NAME)
        self.setWidget(container)

    # GETTERS

    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings
    
    def title_label(self):
        if not self._title_label:
            self._title_label = QLabel('')
            self._title_label.setStyleSheet('color: black; font-weight: bold; font-size: 14pt;')
        return self._title_label
    
    def donate_button(self):
        if not self._donate_button:
            self._donate_button = QPushButton('Donate')
            self._donate_button.setStyleSheet('background-color: blue; color: white; font-weight: bold;')
            self._donate_button.clicked.connect(self.handle_donate_button)
        return self._donate_button
    
    def stacked_panel(self):
        if not self._stacked_panel:
            self._stacked_panel = StackedPanel()
        return self._stacked_panel
    
    def panels(self):
        return self._panels

    # ADDING PANELS

    def add_panel(self, panel, name):
        self.panels()[name] = panel.title()
        self.stacked_panel().add_panel(panel, name)

    def select_panel(self, name):
        self.title_label().setText(self.panels().get(name))
        self.stacked_panel().switch_to(name)

    # EVENT HANDLERS

    def handle_donate_button(self):
        webbrowser.open(DONATE_URL)


import os
import sys

from PySide6.QtGui import QPixmap, QPainter, QColor


def resource_path(relative_path):
    base_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
    return os.path.join(base_path, relative_path)


def version():
    with open(resource_path('mosamaticdesktop/resources/VERSION'), 'r') as f:
        return f.readline().strip()


def is_macos():
    return sys.platform.startswith('darwin')


def icon(parent, icon_type):
    return parent.style().standardIcon(icon_type)


def set_opacity(pixmap, opacity):
    result = QPixmap(pixmap.size())
    result.fill(QColor(0, 0, 0, 0))
    painter = QPainter(result)
    painter.setOpacity(opacity)
    painter.drawPixmap(0, 0, pixmap)
    painter.end()
    return result


from PySide6.QtWidgets import (
    QWidget,
    QDockWidget,
    QTextEdit,
    QPushButton,
    QVBoxLayout,
    QLabel,
)

from mosamaticdesktop.core.utils.logmanagerlistener import LogManagerListener

PANEL_TITLE = 'Output log'
PANEL_NAME = 'logpanel'


class LogPanel(QDockWidget, LogManagerListener):
    def __init__(self):
        super(LogPanel, self).__init__()
        self._title_label = None
        self._text_edit = None
        self.init_layout()

    def title_label(self):
        if not self._title_label:
            self._title_label = QLabel(PANEL_TITLE)
        return self._title_label

    def text_edit(self):
        if not self._text_edit:
            self._text_edit = QTextEdit()
        return self._text_edit
    
    def init_layout(self):
        clear_logs_button = QPushButton('Clear log')
        clear_logs_button.clicked.connect(self.handle_clear_logs_button)
        layout = QVBoxLayout()
        # layout.addWidget(self.title_label())
        layout.addWidget(self.text_edit())
        layout.addWidget(clear_logs_button)
        container = QWidget()
        container.setLayout(layout)
        self.setObjectName(PANEL_NAME)
        self.setWindowTitle(self.title_label().text())
        self.setWidget(container)

    # HELPERS

    def add_line(self, line):
        self.text_edit().insertPlainText(line + '\n')
        self.move_to_end()

    def move_to_end(self):
        cursor = self.text_edit().textCursor()
        cursor.movePosition(cursor.MoveOperation.End)
        self.text_edit().setTextCursor(cursor)
        self.text_edit().ensureCursorVisible()

    # EVENTS

    def handle_clear_logs_button(self):
        self.text_edit().clear()

    # implements(LogManagerListener)
    def new_message(self, message):
        self.add_line(message)


import os

from PySide6.QtWidgets import (
    QMainWindow,
    QStyle,
)
from PySide6.QtGui import (
    QGuiApplication,
    QAction,
    QIcon,
)
from PySide6.QtCore import Qt, QByteArray

from mosamaticdesktop.ui.settings import Settings
from mosamaticdesktop.ui.panels.mainpanel import MainPanel
from mosamaticdesktop.ui.panels.settingspanel import SettingsPanel
from mosamaticdesktop.ui.panels.datapanel import DataPanel
from mosamaticdesktop.ui.panels.logpanel import LogPanel
from mosamaticdesktop.ui.panels.tasks.decompressdicomfilestaskpanel import DecompressDicomFilesTaskPanel
from mosamaticdesktop.ui.panels.tasks.rescaledicomfilestaskpanel import RescaleDicomFilesTaskPanel
from mosamaticdesktop.ui.panels.tasks.segmentmusclefatl3tensorflowtaskpanel import SegmentMuscleFatL3TensorFlowTaskPanel
from mosamaticdesktop.ui.panels.tasks.createpngsfromsegmentationstaskpanel import CreatePngsFromSegmentationsTaskPanel
from mosamaticdesktop.ui.panels.tasks.calculatescorestaskpanel import CalculateScoresTaskPanel
from mosamaticdesktop.ui.panels.pipelines.defaultpipelinepanel import DefaultPipelinePanel
from mosamaticdesktop.ui.dialogs.loaddicomfiledialog import LoadDicomFileDialog
from mosamaticdesktop.ui.dialogs.loadmultidicomfiledialog import LoadMultiDicomFileDialog
from mosamaticdesktop.ui.utils import resource_path, version, is_macos
from mosamaticdesktop.core.utils.logmanager import LogManager
from mosamaticdesktop.core.data.datamanager import DataManager

LOG = LogManager()

WINDOW_TITLE = 'Mosamatc Desktop'
WINDOW_W = 1024
WINDOW_H = 600
ICON_EXIT = QStyle.SP_MessageBoxCritical
ICON_SETTINGS = QStyle.SP_VistaShield
ICONS_DIR_PATH = 'mosamaticdesktop/resources/images/icons'
ICON_FILE_NAME = 'mosamaticdesktop.icns' if is_macos() else 'mosamaticdesktop.ico'


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self._settings = None
        self._data_manager = None
        self._main_panel = None
        self._settings_panel = None
        self._data_panel = None
        self._log_panel = None
        self._decompress_dicom_files_task_panel = None
        self._rescale_dicom_files_task_panel = None
        self._segment_muscle_fat_l3_tensorflow_task_panel = None
        self._create_pngs_from_segmentations_task_panel = None
        self._calculate_scores_task_panel = None
        self._default_pipeline_panel = None
        self._load_dicom_file_dialog = None
        self._load_multi_dicom_file_dialog = None
        self._load_dicom_series_dialog = None
        self._load_multi_dicom_series_dialog = None
        self.init_window()
        self.init_data_manager()

    def init_window(self):
        self.setWindowTitle(f'{WINDOW_TITLE} {version()}')
        self.setWindowIcon(QIcon(resource_path(os.path.join(ICONS_DIR_PATH, ICON_FILE_NAME))))
        if not self.load_geometry_and_state():
            self.set_default_size_and_position()
        self.init_menus()
        self.init_status_bar()
        self.addDockWidget(Qt.DockWidgetArea.TopDockWidgetArea, self.main_panel())
        self.addDockWidget(Qt.DockWidgetArea.BottomDockWidgetArea, self.log_panel())

    def init_menus(self):
        self.init_app_menu()
        # self.init_data_menu()
        self.init_tasks_menu()
        self.init_pipelines_menu()
        if is_macos():            
            self.menuBar().setNativeMenuBar(False)

    def init_app_menu(self):
        open_settings_action = QAction('Settings...', self)
        open_settings_action.triggered.connect(self.handle_open_settings_panel_action)
        exit_action = QAction('Exit', self)
        exit_action.triggered.connect(self.close)
        app_menu = self.menuBar().addMenu('Application')
        app_menu.addAction(open_settings_action)
        app_menu.addAction(exit_action)

    def init_data_menu(self):
        load_dicom_file_action = QAction('Load DICOM file...', self)
        load_dicom_file_action.triggered.connect(self.handle_load_dicom_file_action)
        load_multi_dicom_file_action = QAction('Load multiple DICOM files...', self)
        load_multi_dicom_file_action.triggered.connect(self.handle_load_multi_dicom_action)
        load_dicom_series_action = QAction('Load DICOM series...', self)
        load_dicom_series_action.triggered.connect(self.handle_load_dicom_series_action)
        load_multi_dicom_series_action = QAction('Load multiple DICOM series...', self)
        load_multi_dicom_series_action.triggered.connect(self.handle_load_multi_dicom_series_action)
        open_data_panel_action = QAction('Data manager', self)
        open_data_panel_action.triggered.connect(self.handle_open_data_panel_action)
        data_menu = self.menuBar().addMenu('Data')
        data_menu.addAction(load_dicom_file_action)
        data_menu.addAction(load_multi_dicom_file_action)
        data_menu.addAction(load_dicom_series_action)
        data_menu.addAction(load_multi_dicom_series_action)
        data_menu.addAction(open_data_panel_action)

    def init_tasks_menu(self):
        decompress_dicom_files_task_action = QAction('Decompress DICOM files', self)
        decompress_dicom_files_task_action.triggered.connect(self.handle_decompress_dicom_files_task_action)
        rescale_dicom_files_task_action = QAction('Rescale DICOM files', self)
        rescale_dicom_files_task_action.triggered.connect(self.handle_rescale_dicom_files_task_action)
        segment_muscle_fat_l3_tensorflow_task_action = QAction('Segment muscle and fat L3 (TensorFlow)', self)
        segment_muscle_fat_l3_tensorflow_task_action.triggered.connect(self.handle_segment_muscle_fat_l3_tensorflow_task_action)
        create_pngs_from_segmentations_task_action = QAction('Create PNG images from segmentation masks', self)
        create_pngs_from_segmentations_task_action.triggered.connect(self.handle_create_pngs_from_segmentations_task_action)
        calculate_scores_task_action = QAction('Calculate body composition scores', self)
        calculate_scores_task_action.triggered.connect(self.handle_calculate_scores_task_action)
        tasks_menu = self.menuBar().addMenu('Tasks')
        tasks_menu.addAction(decompress_dicom_files_task_action)
        tasks_menu.addAction(rescale_dicom_files_task_action)
        tasks_menu.addAction(segment_muscle_fat_l3_tensorflow_task_action)
        tasks_menu.addAction(create_pngs_from_segmentations_task_action)
        tasks_menu.addAction(calculate_scores_task_action)

    def init_pipelines_menu(self):
        default_pipeline_action = QAction('Default pipeline', self)
        default_pipeline_action.triggered.connect(self.handle_default_pipeline_action)
        pipelines_menu = self.menuBar().addMenu('Pipelines')
        pipelines_menu.addAction(default_pipeline_action)

    def init_status_bar(self):
        self.set_status('Ready')

    def init_data_manager(self):
        self.data_manager().add_listener(self.data_panel())
        self.data_manager().load_data_from_file(self.settings().get('mainwindow/data_objects'))

    # GETTERS

    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings
    
    def data_manager(self):
        if not self._data_manager:
            self._data_manager = DataManager()
        return self._data_manager
    
    def main_panel(self):
        if not self._main_panel:
            self._main_panel = MainPanel(self)
            self._main_panel.add_panel(self.data_panel(), 'datapanel')
            self._main_panel.add_panel(self.settings_panel(), 'settingspanel')
            self._main_panel.add_panel(self.decompress_dicom_files_task_panel(), 'decompressdicomfilestaskpanel')
            self._main_panel.add_panel(self.rescale_dicom_files_task_panel(), 'rescaledicomfilestaskpanel')
            self._main_panel.add_panel(self.segment_muscle_fat_l3_tensorflow_task_panel(), 'segmentmusclefatl3tensorflowtaskpanel')
            self._main_panel.add_panel(self.create_pngs_from_segmentations_task_panel(), 'createpngsfromsegmentationstaskpanel')
            self._main_panel.add_panel(self.calculate_scores_task_panel(), 'calculatescorestaskpanel')
            self._main_panel.add_panel(self.default_pipeline_panel(), 'defaultpipelinepanel')
            self._main_panel.select_panel('settingspanel')
        return self._main_panel
    
    def settings_panel(self):
        if not self._settings_panel:
            self._settings_panel = SettingsPanel()
        return self._settings_panel
    
    def data_panel(self):
        if not self._data_panel:
            self._data_panel = DataPanel()
        return self._data_panel
    
    def log_panel(self):
        if not self._log_panel:
            self._log_panel = LogPanel()
            LOG.add_listener(self._log_panel)
            from mosamatic.logging import LogManager as MosamaticLogManager
            mosamatic_logger = MosamaticLogManager()
            mosamatic_logger.add_listener(self._log_panel)
        return self._log_panel
    
    def decompress_dicom_files_task_panel(self):
        if not self._decompress_dicom_files_task_panel:
            self._decompress_dicom_files_task_panel = DecompressDicomFilesTaskPanel()
        return self._decompress_dicom_files_task_panel

    def rescale_dicom_files_task_panel(self):
        if not self._rescale_dicom_files_task_panel:
            self._rescale_dicom_files_task_panel = RescaleDicomFilesTaskPanel()
        return self._rescale_dicom_files_task_panel
    
    def segment_muscle_fat_l3_tensorflow_task_panel(self):
        if not self._segment_muscle_fat_l3_tensorflow_task_panel:
            self._segment_muscle_fat_l3_tensorflow_task_panel = SegmentMuscleFatL3TensorFlowTaskPanel()
        return self._segment_muscle_fat_l3_tensorflow_task_panel

    def create_pngs_from_segmentations_task_panel(self):
        if not self._create_pngs_from_segmentations_task_panel:
            self._create_pngs_from_segmentations_task_panel = CreatePngsFromSegmentationsTaskPanel()
        return self._create_pngs_from_segmentations_task_panel
    
    def calculate_scores_task_panel(self):
        if not self._calculate_scores_task_panel:
            self._calculate_scores_task_panel = CalculateScoresTaskPanel()
        return self._calculate_scores_task_panel
    
    def default_pipeline_panel(self):
        if not self._default_pipeline_panel:
            self._default_pipeline_panel = DefaultPipelinePanel()
        return self._default_pipeline_panel
    
    def load_dicom_file_dialog(self):
        if not self._load_dicom_file_dialog:
            self._load_dicom_file_dialog = LoadDicomFileDialog()
        return self._load_dicom_file_dialog
    
    def load_multi_dicom_file_dialog(self):
        if not self._load_multi_dicom_file_dialog:
            self._load_multi_dicom_file_dialog = LoadMultiDicomFileDialog()
        return self._load_multi_dicom_file_dialog
    
    def load_dicom_series_dialog(self):
        pass

    def load_multi_dicom_series_dialog(self):
        pass
    
    # SETTERS

    def set_status(self, message):
        self.statusBar().showMessage(message)

    # EVENT HANDLERS

    def handle_open_settings_panel_action(self):
        self.main_panel().select_panel('settingspanel')

    def handle_open_data_panel_action(self):
        self.main_panel().select_panel('datapanel')

    def handle_load_dicom_file_action(self):
        self.load_dicom_file_dialog().exec()

    def handle_load_multi_dicom_action(self):
        self.load_multi_dicom_file_dialog().exec()

    def handle_load_dicom_series_action(self):
        pass

    def handle_load_multi_dicom_series_action(self):
        pass

    def handle_decompress_dicom_files_task_action(self):
        self.main_panel().select_panel('decompressdicomfilestaskpanel')

    def handle_rescale_dicom_files_task_action(self):
        self.main_panel().select_panel('rescaledicomfilestaskpanel')

    def handle_segment_muscle_fat_l3_tensorflow_task_action(self):
        self.main_panel().select_panel('segmentmusclefatl3tensorflowtaskpanel')

    def handle_create_pngs_from_segmentations_task_action(self):
        self.main_panel().select_panel('createpngsfromsegmentationstaskpanel')

    def handle_calculate_scores_task_action(self):
        self.main_panel().select_panel('calculatescorestaskpanel')

    def handle_default_pipeline_action(self):
        self.main_panel().select_panel('defaultpipelinepanel')

    def showEvent(self, event):
        return super().showEvent(event)

    def closeEvent(self, event):
        self.save_geometry_and_state()
        # Save inputs and parameters of relevant panels
        self.decompress_dicom_files_task_panel().save_inputs_and_parameters()
        self.rescale_dicom_files_task_panel().save_inputs_and_parameters()
        self.segment_muscle_fat_l3_tensorflow_task_panel().save_inputs_and_parameters()
        self.create_pngs_from_segmentations_task_panel().save_inputs_and_parameters()
        self.calculate_scores_task_panel().save_inputs_and_parameters()
        self.default_pipeline_panel().save_inputs_and_parameters()
        # Save data manager objects
        self.settings().set('mainwindow/data_objets', self.data_manager().save_data_to_file())
        return super().closeEvent(event)
    
    # MISCELLANEOUS

    def load_geometry_and_state(self):
        geometry = self.settings().get('mainwindow/geometry')
        state = self.settings().get('mainwindow/state')
        if isinstance(geometry, QByteArray) and self.restoreGeometry(geometry):
            if isinstance(state, QByteArray):
                self.restoreState(state)
            return True
        return False

    def save_geometry_and_state(self):
        self.settings().set('mainwindow/geometry', self.saveGeometry())
        self.settings().set('mainwindow/state', self.saveState())

    def set_default_size_and_position(self):
        self.resize(WINDOW_W, WINDOW_H)
        self.center_window()

    def center_window(self):
        screen = QGuiApplication.primaryScreen().geometry()
        x = (screen.width() - self.geometry().width()) / 2
        y = (screen.height() - self.geometry().height()) / 2
        self.move(int(x), int(y))


import time
import traceback

from PySide6.QtCore import (
    QObject, 
    Signal, 
)

from mosamaticdesktop.core.utils.logmanager import LogManager

LOG = LogManager()


class Worker(QObject):
    progress = Signal(int)
    status = Signal(str)
    finished = Signal()

    def __init__(self, task):
        super(Worker, self).__init__()
        self._task = task

    def run(self):
        try:
            self.status.emit('started')
            self._task.run()
            self.status.emit('done')
            self.finished.emit()
        except Exception as e:
            self.status.emit('failed')
            LOG.error(traceback.format_exc())
            self.finished.emit()


import os

from PySide6.QtWidgets import (
    QLineEdit,
    QPushButton,
    QFileDialog,
    QLabel,
    QVBoxLayout,
)

from mosamaticdesktop.ui.dialogs.dialog import Dialog
from mosamaticdesktop.ui.settings import Settings

WINDOW_TITLE = 'Select file'


class LoadFileDialog(Dialog):
    def __init__(self, parent=None):
        super(LoadFileDialog, self).__init__(parent)
        self._file_path_label = None
        self._file_path_line_edit = None
        self._name_label = None
        self._name_line_edit = None
        self._open_file_select_dialog_button = None
        self._load_button = None
        self.init_layout()

    def file_path_label(self):
        if not self._file_path_label:
            self._file_path_label = QLabel('File path')
        return self._file_path_label
    
    def file_path_line_edit(self):
        if not self._file_path_line_edit:
            self._file_path_line_edit = QLineEdit(placeholderText='Enter file path')
        return self._file_path_line_edit
    
    def name_label(self):
        if not self._name_label:
            self._name_label = QLabel('Name (optional)')
        return self._name_label
    
    def name_line_edit(self):
        if not self._name_line_edit:
            self._name_line_edit = QLineEdit(placeholderText='Enter optional name for data')
        return self._name_line_edit
    
    def open_file_select_dialog_button(self):
        if not self._open_file_select_dialog_button:
            self._open_file_select_dialog_button = QPushButton('Select file')
            self._open_file_select_dialog_button.clicked.connect(self.handle_open_file_select_dialog_button)
        return self._open_file_select_dialog_button
    
    def load_button(self):
        if not self._load_button:
            self._load_button = QPushButton('Load')
            self._load_button.clicked.connect(self.handle_load_button)
            self._load_button.setEnabled(False)
        return self._load_button

    def init_layout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.file_path_label())
        layout.addWidget(self.file_path_line_edit())
        layout.addWidget(self.name_label())
        layout.addWidget(self.name_line_edit())
        layout.addWidget(self.open_file_select_dialog_button())
        layout.addWidget(self.load_button())
        self.setLayout(layout)

    def handle_open_file_select_dialog_button(self):
        self.load_button().setEnabled(False)
        last_directory = Settings().get('last_directory')
        file_path, _ = QFileDialog.getOpenFileName(self, WINDOW_TITLE, dir=last_directory)
        if file_path:
            self.file_path_line_edit().setText(file_path)
            self.load_button().setEnabled(True)
            directory = os.path.dirname(file_path)
            Settings().set('last_directory', directory)

    def handle_load_button(self):
        raise NotImplementedError()

    def clear(self):
        self.file_path_line_edit().setText('')
        self.name_line_edit().setText('')


from mosamaticdesktop.core.data.data import Data
from mosamaticdesktop.core.data.filedata import FileData


class MultiDicomSeriesData(Data, FileData):
    def __init__(self):
        self._dir_path = None
        self._name = None
        self._object = None

    # implements(FileSetData)
    def path(self):
        return self._dir_path
    
    # implements(FileSetData)
    def set_path(self, path):
        self._dir_path = path

    # implements(Data)
    def name(self):
        return self._name
    
    # implements(Data)
    def set_name(self, name):
        self._name = name

    # implements(Data)
    def object(self):
        return self._object
    
    # implements(Data)
    def set_object(self, object):
        self._object = object


from abc import ABC, abstractmethod


# Do not let interfaces inherit from each other. Just use polymorphism
# to have a class implement multiple interfaces
class Data(ABC):

    @abstractmethod
    def name(self):
        pass

    @abstractmethod
    def set_name(self, name):
        pass

    @abstractmethod
    def object(self):
        pass

    @abstractmethod
    def set_object(self, object) -> None:
        pass


import os

from mosamaticdesktop.core.loaders.loader import Loader
from mosamaticdesktop.core.loaders.fileloader import FileLoader
from mosamaticdesktop.core.loaders.dicomfileloader import DicomFileLoader
from mosamaticdesktop.core.data.dicomseriesdata import DicomSeriesData


class MultiDicomSeriesLoader(Loader, FileLoader):
    def __init__(self):
        self._dir_path = None

    # implements(FileSetLoader)
    def path(self):
        return self._dir_path
    
    # implements(FileSetLoader)
    def set_path(self, path):
        self._dir_path = path

    # implements(Loader)
    def load(self):
        # if self.dir_path():
        #     data = DicomSeriesData()
        #     data.set_dir_path(self.dir_path())
        #     object = []
        #     for f in os.listdir(self.dir_path()):
        #         f_path = os.path.join(self.dir_path(), f)
        #         loader = DicomFileLoader()
        #         loader.set_file_path(f_path)
        #         file_data = loader.load() # Returns None if not DICOM
        #         if file_data:
        #             object.append(file_data)
        #     # Sort DICOM objects by instance number
        #     object_sorted = sorted(object, key=lambda item: int(item.object().get('InstanceNumber')))
        #     data.set_object(object_sorted)
        #     return data
        # raise RuntimeError('Directory path not set')
        pass


import json
import importlib

from mosamaticdesktop.core.singleton import singleton
from mosamaticdesktop.core.utils.functions import create_name_with_timestamp


@singleton
class DataManager:
    def __init__(self):
        self._data = {}
        self._listeners = []

    def add(self, data):
        if not data.name():
            data.set_name(create_name_with_timestamp(f'{data.__class__.__name__.lower()}-'))
        if not data.name() in self._data.keys():
            self._data[data.name()] = data
            self.notify_listeners(self._data[data.name()])
        else:
            raise RuntimeError(f'Data object with name "{data.name()}" already added')
    
    def get(self, name):
        if name in self._data.keys():
            return self._data[name]
        return None
    
    def add_listener(self, listener):
        if listener not in self._listeners:
            self._listeners.append(listener)

    def notify_listeners(self, data):
        for listener in self._listeners:
            listener.new_data(data)

    def save_data_to_file(self):
        file_info = {}
        for data in self._data.values():
            file_info[data.name()] = {
                'class_name': data.__class__.__name__,
                'path': data.path(),
            }
        return json.dumps(file_info)

    def load_data_from_file(self, file_info):
        if file_info:
            file_info = json.loads(file_info)
            for name, v in file_info.items():
                class_name = v['class_name']
                path = v['path']
                cls = self.load_class(class_name)
                data = cls()
                data.set_name(name)
                data.set_path(path)
                self.add(data)

    def load_class(self, class_name):
        class_full_name = f'mosamaticdesktop.core.data.{class_name.lower()}.{class_name}'
        module_path, class_name = class_full_name.rsplit(".", 1)
        module = importlib.import_module(module_path)
        cls = getattr(module, class_name)
        return cls


from PySide6.QtWidgets import (
    QTableWidget,
    QTableWidgetItem,
    QHeaderView,
    QVBoxLayout,
    QSizePolicy,
    QPushButton,
    QLabel,
    QMessageBox,
)
from PySide6.QtCore import Qt

from mosamaticdesktop.ui.settings import Settings
from mosamaticdesktop.ui.panels.defaultpanel import DefaultPanel


class TaskPanel(DefaultPanel):
    def __init__(self):
        super(TaskPanel, self).__init__()


from PySide6.QtWidgets import (
    QListWidget,
    QVBoxLayout,
)

from mosamaticdesktop.core.data.datamanagerlistener import DataManagerListener
from mosamaticdesktop.core.utils.logmanager import LogManager
from mosamaticdesktop.ui.panels.defaultpanel import DefaultPanel

LOG = LogManager()

PANEL_TITLE = 'Data manager'
PANEL_NAME = 'datapanel'


class DataPanel(DefaultPanel, DataManagerListener):
    def __init__(self):
        super(DataPanel, self).__init__()
        self.set_title(PANEL_TITLE)
        self._data_list_widget = None
        self.init_layout()

    def data_list_widget(self):
        if not self._data_list_widget:
            self._data_list_widget = QListWidget()
        return self._data_list_widget

    def init_layout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.data_list_widget())
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    # implements(DataManagerListener)
    def new_data(self, data):
        self.data_list_widget().addItem(data.name())
        LOG.info(f'Added new data: {data.name()}')


from PySide6.QtWidgets import (
    QTableWidget,
    QTableWidgetItem,
    QHeaderView,
    QVBoxLayout,
    QSizePolicy,
    QPushButton,
    QLabel,
    QMessageBox,
)
from PySide6.QtCore import Qt

from mosamaticdesktop.ui.settings import Settings
from mosamaticdesktop.ui.panels.defaultpanel import DefaultPanel

PANEL_TITLE = 'Settings'
PANEL_NAME = 'settingspanel'


class SettingsPanel(DefaultPanel):
    def __init__(self):
        super(SettingsPanel, self).__init__()
        self.set_title(PANEL_TITLE)
        self._settings_file_path_label = None
        self._settings = None
        self._settings_table_widget = None
        self._clear_settings_button = None
        self.init_layout()

    def settings_file_path_label(self):
        if not self._settings_file_path_label:
            self._settings_file_path_label = QLabel(f'File path: {self.settings().fileName()}')
        return self._settings_file_path_label

    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings
    
    def settings_table_widget(self):
        if not self._settings_table_widget:
            self._settings_table_widget = QTableWidget()
            self._settings_table_widget.setSortingEnabled(True)
            self._settings_table_widget.horizontalHeader().setVisible(True)
            self._settings_table_widget.verticalHeader().setVisible(False)
            self._settings_table_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
            self._settings_table_widget.setRowCount(len(self.settings().allKeys()))
            self._settings_table_widget.setColumnCount(2)
            self._settings_table_widget.setAlternatingRowColors(True)
            row_index = 0
            for key in self.settings().allKeys():
                self._settings_table_widget.setItem(row_index, 0, QTableWidgetItem(key))
                value = self.settings().get(key)
                if isinstance(value, str) or isinstance(value, int) or isinstance(value, bool) or isinstance(value, float):
                    self._settings_table_widget.setItem(row_index, 1, QTableWidgetItem(str(value)))
                else:
                    self._settings_table_widget.setItem(row_index, 1, QTableWidgetItem('Cannot display (binary data)'))
                row_index += 1
            self._settings_table_widget.resizeColumnsToContents()
            self._settings_table_widget.sortItems(0, Qt.AscendingOrder)
            self._settings_table_widget.setHorizontalHeaderLabels(['NAME', 'VALUE'])
            self._settings_table_widget.horizontalHeader().setSectionResizeMode(QHeaderView.Interactive)
            self._settings_table_widget.horizontalHeader().setSectionResizeMode(1, QHeaderView.Stretch)
            self._settings_table_widget.horizontalHeader().setDefaultAlignment(Qt.AlignLeft | Qt.AlignVCenter)
        return self._settings_table_widget
    
    def clear_settings_button(self):
        if not self._clear_settings_button:
            self._clear_settings_button = QPushButton('Clear settings')
            self._clear_settings_button.setStyleSheet('color: white; background-color: red; font-weight: bold;')
            self._clear_settings_button.clicked.connect(self.handle_clear_settings_button)
        return self._clear_settings_button

    def init_layout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.settings_file_path_label())
        layout.addWidget(self.settings_table_widget())
        layout.addWidget(self.clear_settings_button())
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    def handle_clear_settings_button(self):
        import os
        os.remove(self.settings().fileName())
        QMessageBox.information(self, 'Warning', 'Settings cleared. Please restart application for changes to take effect.')


import os

from PySide6.QtWidgets import (
    QLineEdit,
    QCheckBox,
    QHBoxLayout,
    QVBoxLayout,
    QFormLayout,
    QPushButton,
    QFileDialog,
    QMessageBox,
)
from PySide6.QtCore import (
    QThread, 
    Slot,
)

from mosamaticdesktop.core.utils.logmanager import LogManager
from mosamaticdesktop.ui.panels.taskpanel import TaskPanel
from mosamaticdesktop.ui.settings import Settings
from mosamaticdesktop.ui.utils import is_macos
from mosamaticdesktop.ui.worker import Worker

from mosamatic.tasks import DecompressDicomFilesTask

LOG = LogManager()

PANEL_TITLE = 'Decompress DICOM files'
PANEL_NAME = 'decompressdicomfilestaskpanel'


class DecompressDicomFilesTaskPanel(TaskPanel):
    def __init__(self):
        super(DecompressDicomFilesTaskPanel, self).__init__()
        self.set_title(PANEL_TITLE)
        self._images_dir_line_edit = None
        self._images_dir_select_button = None
        self._output_dir_line_edit = None
        self._output_dir_select_button = None
        self._overwrite_checkbox = None
        self._form_layout = None
        self._run_task_button = None
        self._settings = None
        self._task = None
        self._worker = None
        self._thread = None
        self.init_layout()

    def images_dir_line_edit(self):
        if not self._images_dir_line_edit:
            self._images_dir_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/images_dir'))
        return self._images_dir_line_edit
    
    def images_dir_select_button(self):
        if not self._images_dir_select_button:
            self._images_dir_select_button = QPushButton('Select')
            self._images_dir_select_button.clicked.connect(self.handle_images_dir_select_button)
        return self._images_dir_select_button
    
    def output_dir_line_edit(self):
        if not self._output_dir_line_edit:
            self._output_dir_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/output_dir'))
        return self._output_dir_line_edit
    
    def output_dir_select_button(self):
        if not self._output_dir_select_button:
            self._output_dir_select_button = QPushButton('Select')
            self._output_dir_select_button.clicked.connect(self.handle_output_dir_select_button)
        return self._output_dir_select_button
    
    def overwrite_checkbox(self):
        if not self._overwrite_checkbox:
            self._overwrite_checkbox = QCheckBox('')
            self._overwrite_checkbox.setChecked(self.settings().get_bool(f'{PANEL_NAME}/overwrite', True))
        return self._overwrite_checkbox
    
    def form_layout(self):
        if not self._form_layout:
            self._form_layout = QFormLayout()
            if is_macos():
                self._form_layout.setFieldGrowthPolicy(QFormLayout.ExpandingFieldsGrow)
        return self._form_layout
    
    def run_task_button(self):
        if not self._run_task_button:
            self._run_task_button = QPushButton('Run task')
            self._run_task_button.clicked.connect(self.handle_run_task_button)
        return self._run_task_button
    
    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings
    
    def init_layout(self):
        images_dir_layout = QHBoxLayout()
        images_dir_layout.addWidget(self.images_dir_line_edit())
        images_dir_layout.addWidget(self.images_dir_select_button())
        output_dir_layout = QHBoxLayout()
        output_dir_layout.addWidget(self.output_dir_line_edit())
        output_dir_layout.addWidget(self.output_dir_select_button())
        self.form_layout().addRow('Images directory', images_dir_layout)
        self.form_layout().addRow('Output directory', output_dir_layout)
        self.form_layout().addRow('Overwrite', self.overwrite_checkbox())
        layout = QVBoxLayout()
        layout.addLayout(self.form_layout())
        layout.addWidget(self.run_task_button())
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    # EVENTS

    def handle_images_dir_select_button(self):
        last_directory = self.settings().get('last_directory')
        directory = QFileDialog.getExistingDirectory(dir=last_directory)
        if directory:
            self.images_dir_line_edit().setText(directory)
            self.settings().set('last_directory', directory)

    def handle_output_dir_select_button(self):
        last_directory = self.settings().get('last_directory')
        directory = QFileDialog.getExistingDirectory(dir=last_directory)
        if directory:
            self.output_dir_line_edit().setText(directory)
            self.settings().set('last_directory', directory)

    def handle_run_task_button(self):
        errors = self.check_inputs_and_parameters()
        if len(errors) > 0:
            error_message = 'Following errors were encountered:\n'
            for error in errors:
                error_message += f' - {error}\n'
            QMessageBox.information(self, 'Error', error_message)
        else:
            LOG.info('Running task...')
            self.run_task_button().setEnabled(False)
            self.save_inputs_and_parameters()
            self._task = DecompressDicomFilesTask(
                self.images_dir_line_edit().text(), 
                self.output_dir_line_edit().text(), 
                self.overwrite_checkbox().isChecked()
            )
            # self._task.run()
            self._worker = Worker(self._task)
            self._thread = QThread()
            self._worker.moveToThread(self._thread)
            self._thread.started.connect(self._worker.run)
            self._worker.progress.connect(self.handle_progress)
            self._worker.status.connect(self.handle_status)
            self._worker.finished.connect(self.handle_finished)
            self._worker.finished.connect(self._thread.quit)
            self._worker.finished.connect(self._worker.deleteLater)
            self._thread.finished.connect(self._thread.deleteLater)
            self._thread.start()

    @Slot(int)
    def handle_progress(self, progress):
        LOG.info(f'Progress: {progress} / 100%')

    @Slot(str)
    def handle_status(self, status):
        LOG.info(f'Status: {status}')

    @Slot()
    def handle_finished(self):
        self.run_task_button().setEnabled(True)

    # HELPERS

    def check_inputs_and_parameters(self):
        errors = []
        if self.images_dir_line_edit().text() == '':
            errors.append('Empty images directory path')
        if not os.path.isdir(self.images_dir_line_edit().text()):
            errors.append('Images directory does not exist')
        if self.output_dir_line_edit().text() == '':
            errors.append('Empty output directory path')
        if os.path.isdir(self.output_dir_line_edit().text()) and not self.overwrite_checkbox().isChecked():
            errors.append('Output directory exists but overwrite=False. Please remove output directory first')
        return errors
    
    def save_inputs_and_parameters(self):
        self.settings().set(f'{PANEL_NAME}/images_dir', self.images_dir_line_edit().text())
        self.settings().set(f'{PANEL_NAME}/output_dir', self.output_dir_line_edit().text())
        self.settings().set(f'{PANEL_NAME}/overwrite', self.overwrite_checkbox().isChecked())


from mosamaticdesktop.core.loaders.multidicomfileloader import MultiDicomFileLoader
from mosamaticdesktop.core.data.datamanager import DataManager
from mosamaticdesktop.core.utils.logmanager import LogManager
from mosamaticdesktop.ui.dialogs.loadfilesetdialog import LoadFileSetDialog

LOG = LogManager()

WINDOW_TITLE = 'Load multiple DICOM files'


class LoadMultiDicomFileDialog(LoadFileSetDialog):
    def __init__(self, parent=None):
        super(LoadMultiDicomFileDialog, self).__init__(parent)
        self.setWindowTitle(WINDOW_TITLE)

    def handle_load_button(self):
        loader = MultiDicomFileLoader()
        loader.set_path(self.directory_path_line_edit().text())
        data = loader.load()
        if data:
            data.set_name(self.name_line_edit().text())
            manager = DataManager()
            manager.add(data)
        self.close()